@@ -44,16 +44,9 @@ Matplotlib recognizes the following formats to specify a color:
 All string specifications of color, other than "CN", are case-insensitive.
 """
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-from six.moves import zip
-
-from collections import Sized
+from collections.abc import Sized
 import itertools
 import re
-import warnings
 
 import numpy as np
 import matplotlib.cbook as cbook
@@ -62,15 +55,15 @@ from ._color_data import BASE_COLORS, TABLEAU_COLORS, CSS4_COLORS, XKCD_COLORS
 
 class _ColorMapping(dict):
     def __init__(self, mapping):
-        super(_ColorMapping, self).__init__(mapping)
+        super().__init__(mapping)
         self.cache = {}
 
     def __setitem__(self, key, value):
-        super(_ColorMapping, self).__setitem__(key, value)
+        super().__setitem__(key, value)
         self.cache.clear()
 
     def __delitem__(self, key):
-        super(_ColorMapping, self).__delitem__(key)
+        super().__delitem__(key)
         self.cache.clear()
 
 
@@ -98,7 +91,7 @@ def _sanitize_extrema(ex):
     if ex is None:
         return ex
     try:
-        ret = np.asscalar(ex)
+        ret = ex.item()
     except AttributeError:
         ret = float(ex)
     return ret
@@ -106,13 +99,12 @@ def _sanitize_extrema(ex):
 
 def _is_nth_color(c):
     """Return whether *c* can be interpreted as an item in the color cycle."""
-    return isinstance(c, six.string_types) and re.match(r"\AC[0-9]\Z", c)
+    return isinstance(c, str) and re.match(r"\AC[0-9]\Z", c)
 
 
 def is_color_like(c):
     """Return whether *c* can be interpreted as an RGB(A) color."""
-    # Special-case nth color syntax because it cannot be parsed during
-    # setup.
+    # Special-case nth color syntax because it cannot be parsed during setup.
     if _is_nth_color(c):
         return True
     try:
@@ -180,7 +172,7 @@ def _to_rgba_no_colorcycle(c, alpha=None):
     ``"none"`` (case-insensitive), which always maps to ``(0, 0, 0, 0)``.
     """
     orig_c = c
-    if isinstance(c, six.string_types):
+    if isinstance(c, str):
         if c.lower() == "none":
             return (0., 0., 0., 0.)
         # Named color.
@@ -189,7 +181,7 @@ def _to_rgba_no_colorcycle(c, alpha=None):
             c = _colors_full_map[c.lower()]
         except KeyError:
             pass
-    if isinstance(c, six.string_types):
+    if isinstance(c, str):
         # hex color with no alpha.
         match = re.match(r"\A#[a-fA-F0-9]{6}\Z", c)
         if match:
@@ -255,7 +247,7 @@ def to_rgba_array(c, alpha=None):
     # Note that this occurs *after* handling inputs that are already arrays, as
     # `to_rgba(c, alpha)` (below) is expensive for such inputs, due to the need
     # to format the array in the ValueError message(!).
-    if isinstance(c, six.string_types) and c.lower() == "none":
+    if cbook._str_lower_equal(c, "none"):
         return np.zeros((0, 4), float)
     try:
         return np.array([to_rgba(c, alpha)], float)
@@ -522,8 +514,7 @@ class Colormap(object):
             lut = self._lut.copy()  # Don't let alpha modify original _lut.
 
         if alpha is not None:
-            alpha = min(alpha, 1.0)  # alpha must be between 0 and 1
-            alpha = max(alpha, 0.0)
+            alpha = np.clip(alpha, 0, 1)
             if bytes:
                 alpha = int(alpha * 255)
             if (lut[-1] == 0).all():
@@ -555,7 +546,7 @@ class Colormap(object):
     def set_bad(self, color='k', alpha=None):
         """Set color to be used for masked values.
         """
-        self._rgba_bad = colorConverter.to_rgba(color, alpha)
+        self._rgba_bad = to_rgba(color, alpha)
         if self._isinit:
             self._set_extremes()
 
@@ -563,7 +554,7 @@ class Colormap(object):
         """Set color to be used for low out-of-range values.
            Requires norm.clip = False
         """
-        self._rgba_under = colorConverter.to_rgba(color, alpha)
+        self._rgba_under = to_rgba(color, alpha)
         if self._isinit:
             self._set_extremes()
 
@@ -571,7 +562,7 @@ class Colormap(object):
         """Set color to be used for high out-of-range values.
            Requires norm.clip = False
         """
-        self._rgba_over = colorConverter.to_rgba(color, alpha)
+        self._rgba_over = to_rgba(color, alpha)
         if self._isinit:
             self._set_extremes()
 
@@ -718,7 +709,7 @@ class LinearSegmentedColormap(Colormap):
             raise ValueError('colors must be iterable')
 
         if (isinstance(colors[0], Sized) and len(colors[0]) == 2
-                and not isinstance(colors[0], six.string_types)):
+                and not isinstance(colors[0], str)):
             # List of value, color pairs
             vals, colors = zip(*colors)
         else:
@@ -726,7 +717,7 @@ class LinearSegmentedColormap(Colormap):
 
         cdict = dict(red=[], green=[], blue=[], alpha=[])
         for val, color in zip(vals, colors):
-            r, g, b, a = colorConverter.to_rgba(color)
+            r, g, b, a = to_rgba(color)
             cdict['red'].append((val, r, r))
             cdict['green'].append((val, g, g))
             cdict['blue'].append((val, b, b))
@@ -764,13 +755,9 @@ class LinearSegmentedColormap(Colormap):
                 return dat(1.0 - x)
             return func_r
 
-        data_r = dict()
-        for key, data in six.iteritems(self._segmentdata):
-            if callable(data):
-                data_r[key] = factory(data)
-            else:
-                new_data = [(1.0 - x, y1, y0) for x, y0, y1 in reversed(data)]
-                data_r[key] = new_data
+        data_r = {key: (factory(data) if callable(data) else
+                        [(1.0 - x, y1, y0) for x, y0, y1 in reversed(data)])
+                  for key, data in self._segmentdata.items()}
 
         return LinearSegmentedColormap(name, data_r, self.N, self._gamma)
 
@@ -811,7 +798,7 @@ class ListedColormap(Colormap):
             self.colors = colors
             N = len(colors)
         else:
-            if isinstance(colors, six.string_types):
+            if isinstance(colors, str):
                 self.colors = [colors] * N
                 self.monochrome = True
             elif cbook.iterable(colors):
@@ -830,9 +817,8 @@ class ListedColormap(Colormap):
         Colormap.__init__(self, name, N)
 
     def _init(self):
-        rgba = colorConverter.to_rgba_array(self.colors)
         self._lut = np.zeros((self.N + 3, 4), float)
-        self._lut[:-3] = rgba
+        self._lut[:-3] = to_rgba_array(self.colors)
         self._isinit = True
         self._set_extremes()
 
@@ -955,11 +941,6 @@ class Normalize(object):
             resdat -= vmin
             resdat /= (vmax - vmin)
             result = np.ma.array(resdat, mask=result.mask, copy=False)
-        # Agg cannot handle float128.  We actually only need 32-bit of
-        # precision, but on Windows, `np.dtype(np.longdouble) == np.float64`,
-        # so casting to float32 would lose precision on float64s as well.
-        if result.dtype == np.longdouble:
-            result = result.astype(np.float64)
         if is_scalar:
             result = result[0]
         return result
@@ -1130,7 +1111,8 @@ class SymLogNorm(Normalize):
         """
         Inplace transformation.
         """
-        masked = np.abs(a) > self.linthresh
+        with np.errstate(invalid="ignore"):
+            masked = np.abs(a) > self.linthresh
         sign = np.sign(a[masked])
         log = (self._linscale_adj + np.log(np.abs(a[masked]) / self.linthresh))
         log *= sign * self.linthresh
@@ -1187,8 +1169,8 @@ class SymLogNorm(Normalize):
 
 class PowerNorm(Normalize):
     """
-    Normalize a given value to the ``[0, 1]`` interval with a power-law
-    scaling. This will clip any negative data points to 0.
+    Linearly map a given value to the 0-1 range and then apply
+    a power-law normalization over that range.
     """
     def __init__(self, gamma, vmin=None, vmax=None, clip=False):
         Normalize.__init__(self, vmin, vmax, clip)
@@ -1208,18 +1190,17 @@ class PowerNorm(Normalize):
         elif vmin == vmax:
             result.fill(0)
         else:
-            res_mask = result.data < 0
             if clip:
                 mask = np.ma.getmask(result)
                 result = np.ma.array(np.clip(result.filled(vmax), vmin, vmax),
                                      mask=mask)
             resdat = result.data
             resdat -= vmin
+            resdat[resdat < 0] = 0
             np.power(resdat, gamma, resdat)
             resdat /= (vmax - vmin) ** gamma
 
             result = np.ma.array(resdat, mask=result.mask, copy=False)
-            result[res_mask] = 0
         if is_scalar:
             result = result[0]
         return result
@@ -1241,10 +1222,6 @@ class PowerNorm(Normalize):
         Set *vmin*, *vmax* to min, max of *A*.
         """
         self.vmin = np.ma.min(A)
-        if self.vmin < 0:
-            self.vmin = 0
-            warnings.warn("Power-law scaling on negative values is "
-                          "ill-defined, clamping to 0.")
         self.vmax = np.ma.max(A)
 
     def autoscale_None(self, A):
@@ -1252,10 +1229,6 @@ class PowerNorm(Normalize):
         A = np.asanyarray(A)
         if self.vmin is None and A.size:
             self.vmin = A.min()
-            if self.vmin < 0:
-                self.vmin = 0
-                warnings.warn("Power-law scaling on negative values is "
-                              "ill-defined, clamping to 0.")
         if self.vmax is None and A.size:
             self.vmax = A.max()
 
@@ -1496,8 +1469,7 @@ def hsv_to_rgb(hsv):
     g[idx] = v[idx]
     b[idx] = v[idx]
 
-    # `np.stack([r, g, b], axis=-1)` (numpy 1.10).
-    rgb = np.concatenate([r[..., None], g[..., None], b[..., None]], -1)
+    rgb = np.stack([r, g, b], axis=-1)
 
     if in_ndim == 1:
         rgb.shape = (3,)
@@ -1519,17 +1491,6 @@ def _vector_magnitude(arr):
     return np.sqrt(sum_sq)
 
 
-def _vector_dot(a, b):
-    # things that don't work here:
-    #   * a.dot(b) - fails on masked arrays until 1.10
-    #   * np.ma.dot(a, b) - doesn't mask enough things
-    #   * np.ma.dot(a, b, strict=True) - returns a maskedarray with no mask
-    dot = 0
-    for i in range(a.shape[-1]):
-        dot += a[..., i] * b[..., i]
-    return dot
-
-
 class LightSource(object):
     """
     Create a light source coming from the specified azimuth and elevation.
@@ -1666,7 +1627,7 @@ class LightSource(object):
             completely in shadow and 1 is completely illuminated.
         """
 
-        intensity = _vector_dot(normals, self.direction)
+        intensity = normals.dot(self.direction)
 
         # Apply contrast stretch
         imin, imax = intensity.min(), intensity.max()
